﻿//-----------------------------------------------------------------------
// <copyright file="ColorUtilities.cs" author="Artak Mkrtchyan">
//     Creative Commons - GNU General Public License (http://creativecommons.org/licenses/GPL/2.0/
// </copyright>
// <author>Artak Mkrtchyan</author>
// <email>mkArtak@gmail.com</email>
// <date>15/07/2012</date>
//-----------------------------------------------------------------------
using System;
using System.Linq;
using System.Text;
using System.Windows.Media;

namespace ConsoleHoster.Common.Utilities
{
	public static class ColorUtilities
	{
		public static Color GetColorFromString(string argColor, Color argDefault)
		{
			Color tmpMediaColor;
			try
			{
				if (typeof(Colors).GetProperties(System.Reflection.BindingFlags.Static | System.Reflection.BindingFlags.Public).Where(item => item.PropertyType == typeof(Color) && String.Equals(item.Name, argColor, StringComparison.InvariantCultureIgnoreCase)).Any())
				{
					System.Drawing.Color tmpColor = System.Drawing.Color.FromName(argColor);
					tmpMediaColor = Color.FromArgb(tmpColor.A, tmpColor.R, tmpColor.G, tmpColor.B);
				}
				else
				{
					object tmpParsedColor = ColorConverter.ConvertFromString(argColor.ToLower());
					if (tmpParsedColor != null)
					{
						tmpMediaColor = (Color)tmpParsedColor;
					}
					else
					{
						tmpMediaColor = argDefault;
					}
				}
			}
			catch
			{
				tmpMediaColor = argDefault;
			}

			return tmpMediaColor;
		}

		public static string GetColorNameOrCode(Color color)
		{
			return String.Format("#{0}{1}{2}{3}", GetColorComponentCode(color.A), GetColorComponentCode(color.R), GetColorComponentCode(color.G), GetColorComponentCode(color.B));
		}

		public static Color GetColorFromLinearRange(Color argStartColor, Color argEndColor, int argPercent)
		{
			int tmpAlpha = GetValueInLinearRange(argPercent, argStartColor.A, argEndColor.A);
			int tmpRed = GetValueInLinearRange(argPercent, argStartColor.R, argEndColor.R);
			int tmpGreen = GetValueInLinearRange(argPercent, argStartColor.G, argEndColor.G);
			int tmpBlue = GetValueInLinearRange(argPercent, argStartColor.B, argEndColor.B);

			return Color.FromArgb((byte)tmpAlpha, (byte)tmpRed, (byte)tmpGreen, (byte)tmpBlue);
		}

		private static int GetValueInLinearRange(int argPercent, int argValStart, int argValEnd)
		{
			return argValStart + (argValEnd - argValStart) * argPercent / 100;
		}

		private static string GetColorComponentCode(byte argComponent)
		{
			return PrefixWithChar(String.Format("{0:X}", argComponent), '0', 2);
		}

		private static string PrefixWithChar(string argOriginalText, char argPrefixWith, int argLength)
		{
			StringBuilder sb = new StringBuilder();
			if (!String.IsNullOrEmpty(argOriginalText))
			{
				sb.Append(argOriginalText);
			}

			while (sb.Length < argLength)
			{
				sb.Insert(0, argPrefixWith);
			}

			return sb.ToString();
		}
	}
}