﻿//-----------------------------------------------------------------------
// <copyright file="IConsoleVM.cs" author="Artak Mkrtchyan">
//     Creative Commons - GNU General Public License (http://creativecommons.org/licenses/GPL/2.0/
// </copyright>
// <author>Artak Mkrtchyan</author>
// <email>mkArtak@gmail.com</email>
// <date>11/08/2012</date>
//-----------------------------------------------------------------------

using ConsoleHoster.Common.Model;
using System;

namespace ConsoleHoster.Common.ViewModel
{
	public delegate void ConsoleViewModelEventHandler(IConsoleVM argSender);

	/// <summary>
	/// Represents a contract for working with a project console
	/// </summary>
	public interface IConsoleVM : IConsolePluginApi, IDisposable
	{
		/// <summary>
		/// The event will notify when the project console will be closing
		/// </summary>
		event ConsoleViewModelEventHandler Closing;

		/// <summary>
		/// The event will notify about need to persist changes made to the project
		/// </summary>
		event ConsoleViewModelEventHandler PersistChangesRequest;

		/// <summary>
		/// Closes the console
		/// </summary>
		void Close();

		/// <summary>
		/// Gets or sets the command which is going to execute
		/// </summary>
		string CurrentCommand
		{
			get;
			set;
		}

		/// <summary>
		/// Gets or sets the font settings for current console
		/// </summary>
		FontSettings CurrentFontSettings
		{
			get;
			set;
		}
	}
}