﻿//-----------------------------------------------------------------------
// <copyright file="CommandDataViewModel.cs" author="Artak Mkrtchyan">
//     Creative Commons - GNU General Public License (http://creativecommons.org/licenses/GPL/2.0/
// </copyright>
// <author>Artak Mkrtchyan</author>
// <email>mkArtak@gmail.com</email>
// <date>28/07/2012</date>
//-----------------------------------------------------------------------
using ConsoleHoster.Common.ViewModel;
using ConsoleHoster.Model.Entities;
using System;
using System.Diagnostics;
using System.Windows.Media;

namespace ConsoleHoster.ViewModel.Entities
{
	[DebuggerDisplay("Name={Name}")]
	public class CommandDataViewModel : ViewModelEntityBase<CommandData>
	{
		public CommandDataViewModel(CommandData argModel)
			: base(argModel)
		{

		}

		public CommandDataViewModel()
			: this(new CommandData())
		{

		}

		#region Properties
		public string FullName
		{
			get
			{
				return String.Format("{0}.{1}", this.GroupName, this.Name);
			}
		}

		public string UIFriendlyName
		{
			get
			{
				return this.Name.Replace("_", "__");
			}
		}

		public string Name
		{
			get
			{
				return this.Model.Name;
			}
			set
			{
				if (value != this.Model.Name)
				{
					this.Model.Name = value;
					this.NotifyPropertyChange("Name");
					this.NotifyPropertyChange("FullName");
					this.NotifyPropertyChange("UIFriendlyName");
				}
			}
		}

		public string CommandText
		{
			get
			{
				return this.Model.CommandText;
			}
			set
			{
				if (value != this.Model.CommandText)
				{
					this.Model.CommandText = value;
					this.NotifyPropertyChange("CommandText");
				}
			}
		}

		public bool IsFinal
		{
			get
			{
				return this.Model.IsFinal;
			}
			set
			{
				if (value != this.Model.IsFinal)
				{
					this.Model.IsFinal = value;
					this.NotifyPropertyChange("IsFinal");
				}
			}
		}

		public string GroupName
		{
			get
			{
				return this.Model.GroupName;
			}
			set
			{
				if (value != this.Model.GroupName)
				{
					this.Model.GroupName = value;
					this.NotifyPropertyChange("GroupName");
					this.NotifyPropertyChange("FullName");
				}
			}
		}

		public Color CommandColor
		{
			get
			{
				return this.Model.CommandColor;
			}
			set
			{
				if (value != this.Model.CommandColor)
				{
					this.Model.CommandColor = value;
					this.NotifyPropertyChange("CommandColor");
					this.NotifyPropertyChange("ColorBrush");
				}
			}
		}

		public Brush ColorBrush
		{
			get
			{
				return new SolidColorBrush(this.CommandColor);
			}
		}
		#endregion
	}
}
